@@ -124,7 +124,7 @@ char *amap_get_data_token(char *data, char token) {
 int amap_webupdate_file(char *webfile, char *localfile, int checkversion, int ask) {
   int len = strlen("http://");
   int wlen = strlen(webfile);
-  int port = 80, s, result = 1, datalen = 0, version = -1;
+  int port = 80, s, result = 1, datalen = 0, version = -1, fck;
 int xx = 0;
   unsigned long int ip;
   time_t epoch;
@@ -223,7 +223,7 @@ int xx = 0;
   if (checkversion) {
     if ((f = fopen(localfile, "r")) != NULL) {
       memset(request, 0, AMAP_WEBBUFLEN);
-      fread(request, AMAP_WEBBUFLEN - AMAP_MAXTOKENLEN, 1, f);
+      fck=fread(request, AMAP_WEBBUFLEN - AMAP_MAXTOKENLEN, 1, f);
       version = atoi(amap_get_data_token(request, 'V'));
       fclose(f);
     }
@@ -777,7 +777,14 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
       if ((ptr = amap_index(ptr, ' ')) == NULL)
         amap_error("nmap data line fucked up (is it ipv6 but you did not use the -6 option?) : %s", orig_line);
       *ptr = 0;
-      target->target = strdup(ip);
+      if (index(ip, ':') == NULL)
+        target->target = strdup(ip);
+      else {
+        target->target = malloc(strlen(ip) + 3);
+        strcpy(target->target, "[");
+        strcat(target->target, ip);
+        strcat(target->target, "]");
+      }
       if (opt->ipv6 == 0)
         if (inet_addr(target->target) == -1)
           amap_error("invalid IP address in nmap line: %s : %s", target->target, orig_line);
@@ -1071,24 +1078,43 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
       if (len < 0 && opt->portscanner && errno != 0) {
         if (errno == ECONNREFUSED) {
           if (coms[i].port->skip < 1 && opt->verbose && opt->quiet == 0)
-            amap_warn("Could not connect to %s:%d/%s, disabling port", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            if (coms[i].target != NULL)
+              amap_warn("Could not connect to %s%s%s:%d/%s, disabling port",opt->ipv6 ? "[" : "", coms[i].target->target,opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           coms[i].port->skip = 2;
           shutdown(coms[i].socket, SHUT_RDWR);
           close(coms[i].socket);
           memset(&coms[i], 0, sizeof(amap_struct_coms));
           scaninfo->running--;
-        }
+        } else 
+          if (errno != EAGAIN) { // EAGAIN
+            if (coms[i].port->skip < 1 && opt->verbose && opt->quiet == 0)
+              if (coms[i].target != NULL)
+                amap_warn("Could not connect to %s:%d/%s, disabling port, unknown error: %d", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", errno);
+          } else {
+            if (coms[i].port->ip_prot == AMAP_PROTO_TCP) {
+              printf("Port on %s%s%s:%d/%s is OPEN\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+              if (opt->logfile != NULL) {
+                if (opt->machine_readable)
+                  fprintf(opt->logfile, "%s%s%s:%d:%s:open::%s::\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", AMAP_UFO);
+                else
+                  fprintf(opt->logfile, "Port on %s%s%s:%d/%s is OPEN\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+              }
+              coms[i].port->skip = 1;
+              memset(&coms[i], 0, sizeof(amap_struct_coms));
+              scaninfo->running--;
+            }
+          }
       }
       // oh yeah, we received data!
       if (len > 0 && opt->banner_only) {
-        printf("Banner on %s:%d/%s : %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
+        printf("Banner on %s%s%s:%d/%s : %s\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
         if (opt->logfile != NULL) {
           if (opt->machine_readable) {
-            fprintf(opt->logfile, "%s:%d:%s:%s::%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), AMAP_UFO, amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
+            fprintf(opt->logfile, "%s%s%s:%d:%s:%s::%s:%s:", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), AMAP_UFO, amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
             amap_banner_string(opt->logfile, buf, len);
             fprintf(opt->logfile, "\n");
           } else
-            fprintf(opt->logfile, "Banner on %s:%d/%s : %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
+            fprintf(opt->logfile, "Banner on %s%s%s:%d/%s : %s\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
         }
         coms[i].port->skip = 1;
 #ifdef OPENSSL
@@ -1132,30 +1158,30 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
                     info[0] = 0;
                   if (opt->logfile != NULL) {
                     if (opt->machine_readable) {
-                      fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", response->id, amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
+                      fprintf(opt->logfile, "%s%s%s:%d:%s:%s:%s:%s:%s:", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", response->id, amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                       amap_banner_string(opt->logfile, coms[i].response, coms[i].response_length);
                       fprintf(opt->logfile, "\n");
                     } else {
-                      fprintf(opt->logfile, "Protocol on %s:%d/%s%s%smatches %s", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info, response->id);
+                      fprintf(opt->logfile, "Protocol on %s%s%s:%d/%s%s%smatches %s", opt->ipv6 ? "]" : "", coms[i].target->target, opt->ipv6 ? "[" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info, response->id);
                       if (opt->banner) 
                         fprintf(opt->logfile, " - banner: %s\n", amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                       else
                         fprintf(opt->logfile, "\n");
                       if (opt->dump_all) {
                         snprintf(info, sizeof(info), "(by trigger %s)", coms[i].trigger->id);
-                        fprintf(opt->logfile, "Identified response from %s:%d/%s%s%s:\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
+                        fprintf(opt->logfile, "Identified response from %s%s%s:%d/%s%s%s:\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
                         amap_dump_string(opt->logfile, coms[i].response, coms[i].response_length, 16);
                       }
                     }
                   }
-                  printf("Protocol on %s:%d/%s%s%smatches %s", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info, response->id);
+                  printf("Protocol on %s%s%s:%d/%s%s%smatches %s", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info, response->id);
                   if (opt->banner) 
                     printf(" - banner: %s\n", amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                   else
                     printf("\n");
                   if (opt->dump_all) {
                     snprintf(info, sizeof(info), "(by trigger %s)", coms[i].trigger->id);
-                    printf("Dump of identified response from %s:%d/%s%s%s:\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
+                    printf("Dump of identified response from %s%s%s:%d/%s%s%s:\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
                     amap_dump_string(stdout, coms[i].response, coms[i].response_length, 16);
                   }
                 }
@@ -1171,13 +1197,13 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
             //else
             //  info[0] = 0;
             if (opt->logfile != NULL && !opt->machine_readable) {
-              fprintf(opt->logfile, "Unrecognized response from %s:%d/%s%s%sreceived.\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
+              fprintf(opt->logfile, "Unrecognized response from %s%s%s:%d/%s%s%sreceived.\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
               if (opt->dump_unidentified) {
                 fprintf(opt->logfile, "Please send output + name of the application to %s:\n", AMAP_EMAIL);
                 amap_dump_string(opt->logfile, coms[i].response, coms[i].response_length, 16);
               }
             }
-            printf("Unrecognized response from %s:%d/%s%s%sreceived.\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
+            printf("Unrecognized response from %s%s%s:%d/%s%s%sreceived.\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
             if (opt->dump_unidentified) {
               printf("Please send this output and the name of the application to %s:\n", AMAP_EMAIL);
               amap_dump_string(stdout, coms[i].response, coms[i].response_length, 16);
@@ -1205,13 +1231,13 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
             amap_add_id(coms[i].port, banner);
             if (opt->logfile != NULL) {
               if (opt->machine_readable) {
-                fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", coms[i].port->ids->id, amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
+                fprintf(opt->logfile, "%s%s%s:%d:%s:%s:%s:%s:%s:", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", coms[i].port->ids->id, amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                 amap_banner_string(opt->logfile, coms[i].response, coms[i].response_length);
                 fprintf(opt->logfile, "\n");
               } else
-                fprintf(opt->logfile, "Protocol on %s:%d/%s%smatches %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", coms[i].port->ids->id);
+                fprintf(opt->logfile, "Protocol on %s%s%s:%d/%s%smatches %s\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", coms[i].port->ids->id);
             }
-            printf("Protocol on %s:%d/%s%smatches %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", coms[i].port->ids->id);
+            printf("Protocol on %s%s%s:%d/%s%smatches %s\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", coms[i].port->ids->id);
             coms[i].port->skip = 1;
 #ifdef OPENSSL
             if (coms[i].ssl_enabled)
@@ -1276,10 +1302,7 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
 void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo, amap_struct_options *opt, char *rpc_ptr) {
   struct timeval tv;
   fd_set rfd, wfd;
-  int i;
-  int ii;
-  int res;
-  int error;
+  int i, ii, fck, res, error;
   socklen_t error_len = sizeof(error);
   socklen_t sock_len = sizeof(struct sockaddr);
   int maxfd = -1;
@@ -1315,15 +1338,15 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
       // somethings here ...
         error = 0;
         if (getsockopt(coms[i].socket, SOL_SOCKET, SO_ERROR, &error, (socklen_t *)&error_len) < 0 || error != 0) {
-          if (coms[i].port->skip < 1 && (opt->portscanner == 0 || opt->verbose) && opt->quiet == 0)
-            amap_warn("Could not connect (unreachable) to %s:%d/%s, disabling port (EUNKN)", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+          if (coms[i].port->skip < 1 && opt->portscanner != 1 && opt->verbose && opt->quiet == 0)
+            amap_warn("Could not connect (unreachable) to %s%s%s:%d/%s, disabling port (EUNKN)", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           coms[i].port->skip = 2;
           close(coms[i].socket);
           memset(&coms[i], 0, sizeof(amap_struct_coms));
           scaninfo->running--;
         } else {
           if (opt->verbose > 2)
-            printf("DEBUG: socket %d to %s:%d/%s became READY\n", coms[i].socket, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            printf("DEBUG: socket %d to %s%s%s:%d/%s became READY\n", coms[i].socket, opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           coms[i].active = AMAP_CONNECT_READY;
         }
       // until here
@@ -1371,14 +1394,14 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
             struct sockaddr_in6 sa;
             int t = sizeof(sa);
             getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
-            printf("DEBUG: socket %d/%d to %s:%d/%s is now RETRY CONNECT\n", coms[i].socket, htons(sa.sin6_port), coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            printf("DEBUG: socket %d/%d to %s%s%s:%d/%s is now RETRY CONNECT\n", coms[i].socket, htons(sa.sin6_port), opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           } else
 #endif
           {
             struct sockaddr_in sa;
             int t = sizeof(sa);
             getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
-            printf("DEBUG: socket %d/%d to %s:%d/%s is now RETRY CONNECT\n", coms[i].socket, htons(sa.sin_port), coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            printf("DEBUG: socket %d/%d to %s%s%s:%d/%s is now RETRY CONNECT\n", coms[i].socket, htons(sa.sin_port), opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           }
         }
         shutdown(coms[i].socket, SHUT_RDWR);
@@ -1412,6 +1435,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
         res = 1;
         setsockopt(coms[i].socket, SOL_SOCKET, SO_REUSEADDR, &res, sizeof(res));
         fcntl(coms[i].socket, F_SETFL, O_NONBLOCK);
+printf("this connect\n");
         if ((res = connect(coms[i].socket, (struct sockaddr *) coms[i].sockaddr, coms[i].sockaddr_len)) >= 0)
           coms[i].active = AMAP_CONNECT_READY;
         else {
@@ -1435,7 +1459,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
             getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             dport = htons(sa.sin6_port);
             getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
-            printf("DEBUG: socket %d/%d->%d to %s:%d/%s became RETRY CONNECT\n", coms[i].socket, htons(sa.sin6_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            printf("DEBUG: socket %d/%d->%d to %s%s%s:%d/%s became RETRY CONNECT\n", coms[i].socket, htons(sa.sin6_port), dport, opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           } else
 #endif
           {
@@ -1444,7 +1468,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
             getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             dport = htons(sa.sin_port);
             getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
-            printf("DEBUG: socket %d/%d->%d to %s:%d/%s became RETRY CONNECT\n", coms[i].socket, htons(sa.sin_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            printf("DEBUG: socket %d/%d->%d to %s%s%s:%d/%s became RETRY CONNECT\n", coms[i].socket, htons(sa.sin_port), dport, opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           }
         }
       }
@@ -1457,18 +1481,18 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
       if (coms[i].port->ip_prot == AMAP_PROTO_TCP) {
         shutdown(coms[i].socket, SHUT_RDWR);
         close(coms[i].socket);
-        printf("Port on %s:%d/%s is OPEN\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+        printf("Port on %s%s%s:%d/%s is OPEN\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
         if (opt->logfile != NULL) {
           if (opt->machine_readable)
-            fprintf(opt->logfile, "%s:%d:%s:open::%s::\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", AMAP_UFO);
+            fprintf(opt->logfile, "%s%s%s:%d:%s:open::%s::\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", AMAP_UFO);
           else
-            fprintf(opt->logfile, "Port on %s:%d/%s is OPEN\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+            fprintf(opt->logfile, "Port on %s%s%s:%d/%s is OPEN\n", opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
         }
         coms[i].port->skip = 1;
         memset(&coms[i], 0, sizeof(amap_struct_coms));
         scaninfo->running--;
       } else {
-        write(coms[i].socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
+        fck=write(coms[i].socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
         coms[i].active = AMAP_CONNECT_ACTIVE;
         coms[i].timer = time(NULL);
       }
@@ -1516,7 +1540,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
               SSL_write(coms[i].ssl_socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
 #endif
             } else {
-              write(coms[i].socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
+              fck = write(coms[i].socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
             }
           } else {
             // here we build the RPC packet and send it off
@@ -1543,7 +1567,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
               SSL_write(coms[i].ssl_socket, rpc_ptr, ii);
 #endif
             } else {
-              write(coms[i].socket, rpc_ptr, ii);
+              fck = write(coms[i].socket, rpc_ptr, ii);
             }
           }
           if (opt->verbose > 2) {
@@ -1555,7 +1579,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
               getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
               dport = htons(sa.sin6_port);
               getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
-              printf("DEBUG: socket %d/%d->%d to %s:%d/%s became ACTIVE (trigger %s send)\n", coms[i].socket, ntohs(sa.sin6_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].trigger->id);
+              printf("DEBUG: socket %d/%d->%d to %s%s%s:%d/%s became ACTIVE (trigger %s send)\n", coms[i].socket, ntohs(sa.sin6_port), dport, opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].trigger->id);
             } else
 #endif
             {
@@ -1564,7 +1588,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
               getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
               dport = htons(sa.sin_port);
               getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
-              printf("DEBUG: socket %d/%d->%d to %s:%d/%s became ACTIVE (trigger %s send)\n", coms[i].socket, ntohs(sa.sin_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].trigger->id);
+              printf("DEBUG: socket %d/%d->%d to %s%s%s:%d/%s became ACTIVE (trigger %s send)\n", coms[i].socket, ntohs(sa.sin_port), dport, opt->ipv6 ? "[" : "", coms[i].target->target, opt->ipv6 ? "]" : "", coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].trigger->id);
             }
           }
           coms[i].active = AMAP_CONNECT_ACTIVE;
@@ -1572,7 +1596,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
         }
       } else {
         if (coms[i].port->ip_prot == AMAP_PROTO_UDP)
-          write(coms[i].socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
+          fck = write(coms[i].socket, coms[i].trigger->trigger, coms[i].trigger->trigger_length);
         coms[i].active = AMAP_CONNECT_ACTIVE;
         coms[i].timer = time(NULL);
       }
@@ -1704,7 +1728,7 @@ int amap_scan(int scanmode, amap_struct_targets *targets, amap_struct_triggers *
                     && (scaninfo.scanmode == AMAP_SCANMODE_DEFAULT || (scaninfo.scanmode == AMAP_SCANMODE_SSL && port->ssl) || (scaninfo.scanmode == AMAP_SCANMODE_RPC && port->rpc))))
           ) {
             if (opt->verbose > 1)
-              printf("DEBUG: probing now trigger %s (%d) on %s:%d/%s\n", trigger->id, trigger->ip_prot, target->target, port->port, port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+              printf("DEBUG: probing now trigger %s (%d) on %s%s%s:%d/%s\n", trigger->id, trigger->ip_prot, opt->ipv6 ? "[" : "", target->target, opt->ipv6 ? "]" : "", port->port, port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
             ready_for_next = 0;
 
             // if we do RPC scanning, check if there is a reusable connection to the target port
@@ -1798,7 +1822,6 @@ int amap_scan(int scanmode, amap_struct_targets *targets, amap_struct_triggers *
                   amap_warn("inet_aton failed for %s, removing this target from my list completely", coms[i].target->target);
 #else
                 if (inet_pton(glob_af_inet, coms[i].target->target, glob_addr) < 0) {
-//xxx                if (inet_pton(glob_af_inet, coms[i].target->target, &glob_in) < 0) {
                   amap_warn("inet_pton failed for %s, removing this target from my list completely", coms[i].target->target);
 #endif
                   strcpy(coms[i].target->target, "0.0.0.0");
@@ -1810,7 +1833,7 @@ int amap_scan(int scanmode, amap_struct_targets *targets, amap_struct_triggers *
                     memcpy(glob_sin6.sin6_addr.s6_addr, glob_in6.s6_addr, 16);
                   else
 #endif
-                    glob_sin.sin_addr.s_addr = glob_in.s_addr;
+                  glob_sin.sin_addr.s_addr = glob_in.s_addr;
                   ii = 1;
                   setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &ii, sizeof(ii));
                   fcntl(s, F_SETFL, O_NONBLOCK);
@@ -1867,6 +1890,8 @@ int amap_scan(int scanmode, amap_struct_targets *targets, amap_struct_triggers *
     }
     if (opt->verbose && ii > 0)
       printf("Waiting for timeout on %d connections ...\n", scaninfo.running);
+    if (opt->verbose)
+      opt->verbose--;
     while(scaninfo.running != 0 && i < (opt->timeout_connect + opt->timeout_response + 1) * 10 && ii > 0) {
       if (opt->verbose > 3)
         printf("DEBUG: still running: %d\n", scaninfo.running);
@@ -1969,7 +1994,6 @@ amap_struct_options *amap_main_init() {
   static amap_struct_options lopt;
   
   // INITIALISATION //
-  (void) setvbuf(stdout, NULL, _IONBF, 0);
   memset(&lopt, 0, sizeof(lopt));
   lopt.max_connect_retries = AMAP_MAX_CONNECT_RETRIES;
   lopt.do_scan_ssl = 1;
@@ -2016,6 +2040,8 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
   int  pfrom, pto;
   char *ptr;
   
+  (void) setvbuf(stdout, NULL, _IONBF, 0);
+  
   // VARIABLES VERIFICATION //
   if ((argc < 2) && (opt->file_nmap == NULL) && (opt->update == 0))
     amap_error("no targets to scan defined");
@@ -2128,10 +2154,10 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
       argv[0] = inet_ntoa(glob_in);
 #else
       if ((glob = getaddrinfo(argv[0], NULL, &glob_hints, &glob_result)) != 0)
-        amap_error("can not resolve target: %s (%d/%d)", argv[0], glob, errno);
+        amap_error("can not resolve target (getaddrinfo): %s (%d/%d)", argv[0], glob, errno);
       if ((glob = getnameinfo(glob_result->ai_addr, glob_result->ai_addrlen,
           out, sizeof(out), NULL, 0, NI_NUMERICHOST)) != 0)
-        amap_error("can not resolve address: %s (%d/%d)", argv[0], glob, errno);
+        amap_error("can not resolve address (getnameinfo): %s (%d/%d)", argv[0], glob, errno);
       argv[0] = (char *) &out;
 #endif
     } 
@@ -2139,7 +2165,12 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
       if ((target_tmp = targets = (amap_struct_targets *) malloc(sizeof(amap_struct_targets))) == NULL)
         amap_error("malloc failed");
       memset(target_tmp, 0, sizeof(amap_struct_targets));
-      target_tmp->target = argv[0];
+      if (index(argv[0], ':') == NULL)
+        target_tmp->target = argv[0];
+      else {
+        target_tmp->target = malloc(strlen(argv[0]) + 3);
+        strcpy(target_tmp->target, argv[0]);
+      }
       if (opt->verbose > 1)
         printf("DEBUG: Loading ports for %s ... ", target_tmp->target);
       if ((port_tmp = target_tmp->ports = (amap_struct_ports *) malloc(sizeof(amap_struct_ports))) == NULL)
@@ -2164,7 +2195,14 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
           amap_error("malloc failed");
         target_tmp = (amap_struct_targets *) target_tmp->next;
         memset(target_tmp, 0, sizeof(amap_struct_targets));
-        target_tmp->target = argv[0];
+        if (index(argv[0], ':') == NULL)
+          target_tmp->target = argv[0];
+        else {
+          target_tmp->target = malloc(strlen(argv[0]) + 3);
+          strcpy(target_tmp->target, "[");
+          strcat(target_tmp->target, argv[0]);
+          strcat(target_tmp->target, "]");
+        }
         if (opt->verbose > 1)
           printf("DEBUG: Loading ports for %s ... ", target_tmp->target);
         if ((port_tmp = target_tmp->ports = (amap_struct_ports *) malloc(sizeof(amap_struct_ports))) == NULL)
@@ -2266,8 +2304,11 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
   else if (opt->banner_only)
     printf("BANNER mode\n\n");
   else
-    printf("MAPPING mode\n\n");
+    printf("APPLICATION MAPPING mode\n\n");
   
+  if (opt->portscanner && opt->cmd_proto == AMAP_PROTO_UDP)
+    printf("Warning: UDP port scanning is highly unreliable against Linux, Cisco and other systems that throttle ICMP error messages\n\n");
+
   if (amap_scan(AMAP_SCANMODE_DEFAULT, targets, triggers, responses, opt) == 0)
     amap_error("nothing to scan - no open ports in nmap output file");
 
@@ -2289,14 +2330,14 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
       while(port_tmp != NULL) {
         if (port_tmp->ids == NULL && (opt->quiet == 0 || port_tmp->skip < 2)) {
           i++;
-          printf(" %s:%d/%s", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+          printf(" %s%s%s:%d/%s", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           if (opt->logfile != NULL) {
             if (opt->machine_readable) {
-              fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(port_tmp->skip), port_tmp->ssl ? "SSL" : "", AMAP_UFO, amap_printable_banner_string((char *)port_tmp->unknown_response, port_tmp->unknown_response_length, banner, sizeof(banner)));
+              fprintf(opt->logfile, "%s%s%s:%d:%s:%s:%s:%s:%s:", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(port_tmp->skip), port_tmp->ssl ? "SSL" : "", AMAP_UFO, amap_printable_banner_string((char *)port_tmp->unknown_response, port_tmp->unknown_response_length, banner, sizeof(banner)));
               amap_banner_string(opt->logfile, port_tmp->unknown_response, port_tmp->unknown_response_length);
               fprintf(opt->logfile, "\n");
             } else
-              fprintf(opt->logfile, " %s:%d/%s", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
+              fprintf(opt->logfile, " %s%s%s:%d/%s", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           }
         }
         port_tmp = (amap_struct_ports*) port_tmp->next;
@@ -2331,12 +2372,20 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
         port_tmp = target_tmp->ports;
         while(port_tmp != NULL) {
           if ((port_tmp->skip == 0 || port_tmp->skip == 3) && port_tmp->ip_prot == AMAP_PROTO_UDP) {
-            printf("Port on %s:%d/udp is OPEN\n", target_tmp->target, port_tmp->port);
+            printf("Port on %s%s%s:%d/udp is OPEN\n", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port);
             if (opt->logfile != NULL) {
               if (opt->machine_readable)
-                fprintf(opt->logfile, "%s:%d:%s:open::%s::\n", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", AMAP_UFO);
+                fprintf(opt->logfile, "%s%s%s:%d:%s:open::%s::\n", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", AMAP_UFO);
               else
-                fprintf(opt->logfile, "Port on %s:%d/udp is OPEN\n", target_tmp->target, port_tmp->port);
+                fprintf(opt->logfile, "Port on %s%s%s:%d/udp is OPEN\n", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port);
+            }
+          } else if ((port_tmp->skip == 4) && port_tmp->ip_prot == AMAP_PROTO_UDP) {
+            printf("Port on %s%s%s:%d/udp is OPEN or FILTERED\n", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port);
+            if (opt->logfile != NULL) {
+              if (opt->machine_readable)
+                fprintf(opt->logfile, "%s%s%s:%d:%s:open or filtered::%s::\n", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", AMAP_UFO);
+              else
+                fprintf(opt->logfile, "Port on %s%s%s:%d/udp is OPEN or FILTERED\n", opt->ipv6 ? "[" : "", target_tmp->target, opt->ipv6 ? "]" : "", port_tmp->port);
             }
           }
           port_tmp = (amap_struct_ports*) port_tmp->next;
