@@ -73,7 +73,7 @@ int main(int argc, char *argv[])
   int dont_exit_when_response = 0;
   long int connect_delay = 1;
   long int close_delay = 250;
-  char *str = NULL, *str_old;
+  unsigned char *str = NULL, *str_old;
   int str_len = 0, str_len_old = 0;
   int i, j;
   int s;
